@@ -171,6 +171,19 @@ void ZPalette::setDefaultRules(ZPalette &p) {
              { p.Publish, "lineedit.focused.fg", "window.default.lineedit.focused.fg" },
              { p.Publish, "lineedit.disabled.bg", "window.default.lineedit.disabled.bg" },
              { p.Publish, "lineedit.disabled.fg", "window.default.lineedit.disabled.fg" },
+
+             { p.Publish, "textedit.bg", "window.default.textedit.bg" },
+             { p.Publish, "textedit.fg", "window.default.textedit.fg" },
+             { p.Publish, "textedit.focused.bg", "window.default.textedit.focused.bg" },
+             { p.Publish, "textedit.focused.fg", "window.default.textedit.focused.fg" },
+             { p.Publish, "textedit.disabled.bg", "window.default.textedit.disabled.bg" },
+             { p.Publish, "textedit.disabled.fg", "window.default.textedit.disabled.fg" },
+             { p.Publish, "textedit.selected.bg", "window.default.textedit.selected.bg" },
+             { p.Publish, "textedit.selected.fg", "window.default.textedit.selected.fg" },
+             { p.Publish, "textedit.linenumber.bg", "window.default.textedit.linenumber.bg" },
+             { p.Publish, "textedit.linenumber.fg", "window.default.textedit.linenumber.fg" },
+             { p.Publish, "textedit.focused.linenumber.bg", "window.default.textedit.focused.linenumber.bg" },
+             { p.Publish, "textedit.focused.linenumber.fg", "window.default.textedit.focused.linenumber.fg" },
         }},
 
         {{ QStringLiteral("window"), QStringLiteral("dialog") }, {
@@ -231,6 +244,19 @@ void ZPalette::setDefaultRules(ZPalette &p) {
              { p.Publish, "lineedit.focused.fg", "window.gray.lineedit.focused.fg" },
              { p.Publish, "lineedit.disabled.bg", "window.gray.lineedit.disabled.bg" },
              { p.Publish, "lineedit.disabled.fg", "window.gray.lineedit.disabled.fg" },
+
+             { p.Publish, "textedit.bg", "window.gray.textedit.bg" },
+             { p.Publish, "textedit.fg", "window.gray.textedit.fg" },
+             { p.Publish, "textedit.focused.bg", "window.gray.textedit.focused.bg" },
+             { p.Publish, "textedit.focused.fg", "window.gray.textedit.focused.fg" },
+             { p.Publish, "textedit.disabled.bg", "window.gray.textedit.disabled.bg" },
+             { p.Publish, "textedit.disabled.fg", "window.gray.textedit.disabled.fg" },
+             { p.Publish, "textedit.selected.bg", "window.gray.textedit.selected.bg" },
+             { p.Publish, "textedit.selected.fg", "window.gray.textedit.selected.fg" },
+             { p.Publish, "textedit.linenumber.bg", "window.gray.textedit.linenumber.bg" },
+             { p.Publish, "textedit.linenumber.fg", "window.gray.textedit.linenumber.fg" },
+             { p.Publish, "textedit.focused.linenumber.bg", "window.gray.textedit.focused.linenumber.bg" },
+             { p.Publish, "textedit.focused.linenumber.fg", "window.gray.textedit.focused.linenumber.fg" },
        }},
 
        {{ QStringLiteral("window"), QStringLiteral("cyan") }, {
@@ -291,6 +317,19 @@ void ZPalette::setDefaultRules(ZPalette &p) {
             { p.Publish, "lineedit.focused.fg", "window.cyan.lineedit.focused.fg" },
             { p.Publish, "lineedit.disabled.bg", "window.cyan.lineedit.disabled.bg" },
             { p.Publish, "lineedit.disabled.fg", "window.cyan.lineedit.disabled.fg" },
+
+            { p.Publish, "textedit.bg", "window.cyan.textedit.bg" },
+            { p.Publish, "textedit.fg", "window.cyan.textedit.fg" },
+            { p.Publish, "textedit.focused.bg", "window.cyan.textedit.focused.bg" },
+            { p.Publish, "textedit.focused.fg", "window.cyan.textedit.focused.fg" },
+            { p.Publish, "textedit.disabled.bg", "window.cyan.textedit.disabled.bg" },
+            { p.Publish, "textedit.disabled.fg", "window.cyan.textedit.disabled.fg" },
+            { p.Publish, "textedit.selected.bg", "window.cyan.textedit.selected.bg" },
+            { p.Publish, "textedit.selected.fg", "window.cyan.textedit.selected.fg" },
+            { p.Publish, "textedit.linenumber.bg", "window.cyan.textedit.linenumber.bg" },
+            { p.Publish, "textedit.linenumber.fg", "window.cyan.textedit.linenumber.fg" },
+            { p.Publish, "textedit.focused.linenumber.bg", "window.cyan.textedit.focused.linenumber.bg" },
+            { p.Publish, "textedit.focused.linenumber.fg", "window.cyan.textedit.focused.linenumber.fg" },
        }},
 
     });
@@ -361,6 +400,18 @@ ZPalette ZPalette::classic() {
         { "window.default.lineedit.focused.fg", Colors::brightWhite},
         { "window.default.lineedit.disabled.bg", Colors::darkGray},
         { "window.default.lineedit.disabled.fg", Colors::lightGray},
+        { "window.default.textedit.bg", Colors::lightGray},
+        { "window.default.textedit.fg", Colors::black},
+        { "window.default.textedit.focused.bg", Colors::lightGray},
+        { "window.default.textedit.focused.fg", Colors::brightWhite},
+        { "window.default.textedit.disabled.bg", Colors::darkGray},
+        { "window.default.textedit.disabled.fg", Colors::lightGray},
+        { "window.default.textedit.selected.bg", Colors::brightWhite},
+        { "window.default.textedit.selected.fg", Colors::darkGray},
+        { "window.default.textedit.linenumber.bg", Colors::darkGray},
+        { "window.default.textedit.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
+        { "window.default.textedit.focused.linenumber.bg", Colors::darkGray},
+        { "window.default.textedit.focused.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
 
         { "window.gray.bg", Colors::lightGray},
         { "window.gray.frame.focused.fg", Colors::brightWhite},
@@ -408,6 +459,18 @@ ZPalette ZPalette::classic() {
         { "window.gray.lineedit.focused.fg", Colors::brightWhite},
         { "window.gray.lineedit.disabled.bg", Colors::darkGray},
         { "window.gray.lineedit.disabled.fg", Colors::lightGray},
+        { "window.gray.textedit.bg", Colors::blue},
+        { "window.gray.textedit.fg", Colors::brightWhite},
+        { "window.gray.textedit.focused.bg", Colors::green},
+        { "window.gray.textedit.focused.fg", Colors::brightWhite},
+        { "window.gray.textedit.disabled.bg", Colors::darkGray},
+        { "window.gray.textedit.disabled.fg", Colors::lightGray},
+        { "window.gray.textedit.selected.bg", Colors::brightWhite},
+        { "window.gray.textedit.selected.fg", Colors::darkGray},
+        { "window.gray.textedit.linenumber.bg", { 0, 0, 0x80}},
+        { "window.gray.textedit.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
+        { "window.gray.textedit.focused.linenumber.bg", { 0, 0x80, 0}},
+        { "window.gray.textedit.focused.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
 
         { "window.cyan.bg", Colors::cyan},
         { "window.cyan.frame.focused.fg", Colors::brightWhite},
@@ -455,7 +518,18 @@ ZPalette ZPalette::classic() {
         { "window.cyan.lineedit.focused.fg", Colors::brightWhite},
         { "window.cyan.lineedit.disabled.bg", Colors::darkGray},
         { "window.cyan.lineedit.disabled.fg", Colors::lightGray},
-
+        { "window.cyan.textedit.bg", Colors::blue},
+        { "window.cyan.textedit.fg", Colors::brightWhite},
+        { "window.cyan.textedit.focused.bg", Colors::green},
+        { "window.cyan.textedit.focused.fg", Colors::brightWhite},
+        { "window.cyan.textedit.disabled.bg", Colors::darkGray},
+        { "window.cyan.textedit.disabled.fg", Colors::lightGray},
+        { "window.cyan.textedit.selected.bg", Colors::brightWhite},
+        { "window.cyan.textedit.selected.fg", Colors::darkGray},
+        { "window.cyan.textedit.linenumber.bg", { 0, 0, 0x80}},
+        { "window.cyan.textedit.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
+        { "window.cyan.textedit.focused.linenumber.bg", { 0, 0x80, 0}},
+        { "window.cyan.textedit.focused.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
     });
 
     setDefaultRules(p);
@@ -527,6 +601,18 @@ ZPalette ZPalette::black() {
         { "window.default.lineedit.focused.fg", Colors::brightWhite},
         { "window.default.lineedit.disabled.bg", Colors::darkGray},
         { "window.default.lineedit.disabled.fg", Colors::lightGray},
+        { "window.default.textedit.bg", Colors::lightGray},
+        { "window.default.textedit.fg", Colors::black},
+        { "window.default.textedit.focused.bg", Colors::lightGray},
+        { "window.default.textedit.focused.fg", Colors::brightWhite},
+        { "window.default.textedit.disabled.bg", Colors::darkGray},
+        { "window.default.textedit.disabled.fg", Colors::lightGray},
+        { "window.default.textedit.selected.bg", Colors::brightWhite},
+        { "window.default.textedit.selected.fg", Colors::darkGray},
+        { "window.default.textedit.linenumber.bg", { 0x22, 0x22, 0x22}},
+        { "window.default.textedit.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
+        { "window.default.textedit.focused.linenumber.bg", { 0x22, 0x22, 0x22}},
+        { "window.default.textedit.focused.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
 
         { "window.gray.bg", Colors::lightGray},
         { "window.gray.frame.focused.fg", Colors::brightWhite},
@@ -574,6 +660,18 @@ ZPalette ZPalette::black() {
         { "window.gray.lineedit.focused.fg", Colors::brightWhite},
         { "window.gray.lineedit.disabled.bg", Colors::darkGray},
         { "window.gray.lineedit.disabled.fg", Colors::lightGray},
+        { "window.gray.textedit.bg", Colors::black},
+        { "window.gray.textedit.fg", Colors::brightWhite},
+        { "window.gray.textedit.focused.bg", Colors::green},
+        { "window.gray.textedit.focused.fg", Colors::brightWhite},
+        { "window.gray.textedit.disabled.bg", Colors::darkGray},
+        { "window.gray.textedit.disabled.fg", Colors::lightGray},
+        { "window.gray.textedit.selected.bg", Colors::brightWhite},
+        { "window.gray.textedit.selected.fg", Colors::darkGray},
+        { "window.gray.textedit.linenumber.bg", { 0x22, 0x22, 0x22}},
+        { "window.gray.textedit.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
+        { "window.gray.textedit.focused.linenumber.bg", { 0, 0x80, 0}},
+        { "window.gray.textedit.focused.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
 
         { "window.cyan.bg", Colors::cyan},
         { "window.cyan.frame.focused.fg", Colors::brightWhite},
@@ -621,7 +719,18 @@ ZPalette ZPalette::black() {
         { "window.cyan.lineedit.focused.fg", Colors::brightWhite},
         { "window.cyan.lineedit.disabled.bg", Colors::darkGray},
         { "window.cyan.lineedit.disabled.fg", Colors::lightGray},
-
+        { "window.cyan.textedit.bg", Colors::black},
+        { "window.cyan.textedit.fg", Colors::brightWhite},
+        { "window.cyan.textedit.focused.bg", Colors::green},
+        { "window.cyan.textedit.focused.fg", Colors::brightWhite},
+        { "window.cyan.textedit.disabled.bg", Colors::darkGray},
+        { "window.cyan.textedit.disabled.fg", Colors::lightGray},
+        { "window.cyan.textedit.selected.bg", Colors::brightWhite},
+        { "window.cyan.textedit.selected.fg", Colors::darkGray},
+        { "window.cyan.textedit.linenumber.bg", { 0x22, 0x22, 0x22}},
+        { "window.cyan.textedit.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
+        { "window.cyan.textedit.focused.linenumber.bg", { 0,    0x80,    0}},
+        { "window.cyan.textedit.focused.linenumber.fg", { 0xdd, 0xdd, 0xdd}},
     });
 
     setDefaultRules(p);
