@@ -619,6 +619,13 @@ bool ZTextLineRef::isValid() const {
     return _index != -1;
 }
 
+namespace {
+    struct HighlightSortHelper {
+        int run;
+        const ZFormatRange *ptr;
+    };
+}
+
 void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, ZTextStyle formattingChars,
                         const QVector<ZFormatRange> &ranges) const {
     if (_layout) {
@@ -629,18 +636,66 @@ void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, Z
         ZPainter painterClipped = painter.translateAndClip({0, 0, pos.x() + ld.pos.x() + ld.maxWidth,
                                                             pos.y() + ld.pos.y() + 1});
 
+        // Prepare to use binary search to find first text run that a format range intersects
+        std::vector<int> textRunStarts;
+        textRunStarts.reserve(ld.textRuns.size());
+        for (int i = 0; i < ld.textRuns.size(); i++) {
+            textRunStarts.push_back(ld.textRuns[i].offset);
+        }
+
+        // match each format range to all text runs that intersect it.
+        // This can add multiple entries per original format range.
+        std::vector<HighlightSortHelper> partitionedRanges;
+        partitionedRanges.reserve(ranges.size()); // enough if no format range intersects multiple text runs
+
+        for (int i = 0; i < ranges.size(); i++) {
+            const ZFormatRange &formatRange = ranges[i];
+            const auto formatRangeLength = formatRange.length();
+            if (formatRangeLength <= 0) {
+                continue;
+            }
+            auto formatRangeStart = formatRange.start();
+            int formatRangeEnd = formatRangeStart + formatRangeLength;
+
+            if (formatRangeEnd < 0 || formatRangeStart >= ld.endIndex) {
+                continue;
+            }
+
+            if (formatRangeStart < ld.offset) {
+                formatRangeStart = ld.offset;
+            }
+
+            auto lb = std::upper_bound(textRunStarts.begin(), textRunStarts.end(), formatRangeStart);
+            lb--; // safe as formatRangeStart >= ld.offset and *textRunStarts.begin() == ld.offset
+            int textRunIndex = lb - textRunStarts.begin();
+            for (; textRunIndex < ld.textRuns.size(); textRunIndex++) {
+                partitionedRanges.push_back(HighlightSortHelper{textRunIndex, &formatRange});
+                if (ld.textRuns[textRunIndex].endIndex >= formatRangeEnd) {
+                    break;
+                }
+            }
+        }
+
+        // Sort by run index, so that the rendering code can iterate over the format ranges in O(1)
+        std::stable_sort(partitionedRanges.begin(), partitionedRanges.end(),
+                         [] (const HighlightSortHelper &lhs, const HighlightSortHelper &rhs) {
+            return lhs.run < rhs.run;
+        });
+
+
+        auto nextFormatRange = partitionedRanges.begin();
         for (int i = 0; i < ld.textRuns.size(); i++) {
             const ZTextLayoutPrivate::TextRun &run = ld.textRuns[i];
             if (run.type == ZTextLayoutPrivate::TextRun::COPY) {
                 painterClipped.writeWithAttributes(pos.x() + ld.pos.x() + run.x, pos.y() + ld.pos.y(),
                                                    p->text.mid(run.offset, run.endIndex - run.offset),
                                                    color.foregroundColor(), color.backgroundColor(), color.attributes());
-                for (const ZFormatRange &formatRange: ranges) {
-                    if (formatRange.length() <= 0) {
-                        continue;
-                    }
-                    int formatRangeEnd = formatRange.start() + formatRange.length();
-                    if (formatRange.start() <= run.offset && formatRangeEnd > run.offset) {
+                for (; nextFormatRange != partitionedRanges.end() && nextFormatRange->run == i; nextFormatRange++) {
+                    const ZFormatRange &formatRange = *nextFormatRange->ptr;
+                    const auto formatRangeLength = formatRange.length();
+                    const auto formatRangeStart = formatRange.start();
+                    int formatRangeEnd = formatRangeStart + formatRangeLength;
+                    if (formatRangeStart <= run.offset && formatRangeEnd > run.offset) {
                         // selection ends in run
                         // make sure it's not an invalid position
                         while (formatRangeEnd < run.endIndex
@@ -651,9 +706,9 @@ void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, Z
                                                     p->text.mid(run.offset, std::min(run.endIndex, formatRangeEnd) - run.offset),
                                                     formatRange.format().foregroundColor(), formatRange.format().backgroundColor(),
                                                     formatRange.format().attributes());
-                    } else if (formatRange.start() > run.offset && formatRange.start() < run.endIndex) {
+                    } else if (formatRangeStart > run.offset && formatRangeStart < run.endIndex) {
                         // selection starts in run
-                        int start = formatRange.start();
+                        int start = formatRangeStart;
                         // make sure it's not an invalid position
                         while (start > 0 && p->columns[start - 1] == p->columns[start]) {
                             start--;
@@ -675,12 +730,12 @@ void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, Z
                 }
             } else if (run.type == ZTextLayoutPrivate::TextRun::TAB) {
                 const ZFormatRange *range = nullptr;
-                for (const ZFormatRange &formatRange: ranges) {
-                    if (formatRange.length() <= 0) {
-                        continue;
-                    }
-                    int formatRangeEnd = formatRange.start() + formatRange.length();
-                    if (run.offset >= formatRange.start() && run.offset < formatRangeEnd) {
+                for (; nextFormatRange != partitionedRanges.end() && nextFormatRange->run == i; nextFormatRange++) {
+                    const ZFormatRange &formatRange = *nextFormatRange->ptr;
+                    const auto formatRangeLength = formatRange.length();
+                    const auto formatRangeStart = formatRange.start();
+                    int formatRangeEnd = formatRangeStart + formatRangeLength;
+                    if (run.offset >= formatRangeStart && run.offset < formatRangeEnd) {
                         range = &formatRange;
                     }
                 }
@@ -723,12 +778,12 @@ void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, Z
                 painterClipped.writeWithAttributes(pos.x() + ld.pos.x() + run.x, pos.y() + ld.pos.y(),
                                                    QString(run.width, ch),
                                                    style.foregroundColor(), style.backgroundColor(), style.attributes());
-                for (const ZFormatRange& formatRange: ranges) {
-                    if (formatRange.length() <= 0) {
-                        continue;
-                    }
-                    int formatRangeEnd = formatRange.start() + formatRange.length();
-                    if (formatRange.start() <= run.offset && formatRangeEnd > run.offset) {
+                for (; nextFormatRange != partitionedRanges.end() && nextFormatRange->run == i; nextFormatRange++) {
+                    const ZFormatRange &formatRange = *nextFormatRange->ptr;
+                    const auto formatRangeLength = formatRange.length();
+                    const auto formatRangeStart = formatRange.start();
+                    int formatRangeEnd = formatRangeStart + formatRangeLength;
+                    if (formatRangeStart <= run.offset && formatRangeEnd > run.offset) {
                         // selection ends in run
                         style = formatRange.formattingChar();
                         if (highlightingTrailingWhitespace) {
@@ -737,7 +792,7 @@ void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, Z
                         painterClipped.writeWithAttributes(pos.x() + ld.pos.x() + run.x, pos.y() + ld.pos.y(),
                                                            QString(std::min(run.width, formatRangeEnd - run.offset), ch),
                                                            style.foregroundColor(), style.backgroundColor(), style.attributes());
-                    } else if (formatRange.start() > run.offset && formatRange.start() < run.endIndex) {
+                    } else if (formatRangeStart > run.offset && formatRangeStart < run.endIndex) {
                         // selection starts in run
                         style = formatRange.formattingChar();
                         if (highlightingTrailingWhitespace) {
@@ -745,7 +800,7 @@ void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, Z
                         }
                         painterClipped.writeWithAttributes(pos.x() + ld.pos.x() + p->columns[formatRange.start() - 1],
                                                            pos.y() + ld.pos.y(),
-                                                           QString(std::min(run.endIndex, formatRangeEnd) - formatRange.start(), ch),
+                                                           QString(std::min(run.endIndex, formatRangeEnd) - formatRangeStart, ch),
                                                            style.foregroundColor(), style.backgroundColor(), style.attributes());
                     }
                 }
@@ -772,13 +827,13 @@ void ZTextLineRef::draw(ZPainter painter, const QPoint &pos, ZTextStyle color, Z
                 ZColor fg = color.backgroundColor();
                 ZColor bg = color.foregroundColor();
                 ZTextAttributes attr = color.attributes();
-                for (const ZFormatRange& formatRange: ranges) {
-                    if (formatRange.length() <= 0) {
-                        continue;
-                    }
-                    int formatRangeEnd = formatRange.start() + formatRange.length();
-                    if ((formatRange.start() <= run.offset && formatRangeEnd > run.offset)
-                            || (formatRange.start() > run.offset && formatRange.start() < run.endIndex)) {
+                for (; nextFormatRange != partitionedRanges.end() && nextFormatRange->run == i; nextFormatRange++) {
+                    const ZFormatRange &formatRange = *nextFormatRange->ptr;
+                    const auto formatRangeLength = formatRange.length();
+                    const auto formatRangeStart = formatRange.start();
+                    int formatRangeEnd = formatRangeStart + formatRangeLength;
+                    if ((formatRangeStart <= run.offset && formatRangeEnd > run.offset)
+                            || (formatRangeStart > run.offset && formatRangeStart < run.endIndex)) {
                         fg = formatRange.format().backgroundColor();
                         bg = formatRange.format().foregroundColor();
                         attr = formatRange.format().attributes();
